#include <stdio.h>
#include <cstdlib>
#include <string.h>
#include "global.h"
#include "vector.h"
#include <GL/glut.h>
#include <fstream>
#include "Bitmap.h"
#include <olectl.h>

#define MAX_IMAGEFILENAME_LENGTH 100
#define MAX_TEXCOUNT 100

//// Include the libraries
//#pragma comment (lib, "EaseLib\\EaseCore.lib")
//#pragma comment (lib, "EaseLib\\EasePCB.lib")
//#pragma comment (lib, "EaseLib\\EaseImport.lib")

#define PI 3.1415926535897932384626433832795

typedef struct textureInfo_s {
	char name[MAX_IMAGEFILENAME_LENGTH];
	unsigned int texID;
	int width;
	int height;
	bool alpha;
	int colsize;
} TexInfo;

/* MD2 header */
struct md2_header_t	{
  int ident;                  /* magic number: "IDP2" */
  int version;                /* version: must be 8 */

  int skinwidth;              /* texture width */
  int skinheight;             /* texture height */

  int framesize;              /* size in bytes of a frame */

  int num_skins;              /* number of skins */
  int num_vertices;           /* number of vertices per frame */
  int num_st;                 /* number of texture coordinates */
  int num_tris;               /* number of triangles */
  int num_glcmds;             /* number of opengl commands */
  int num_frames;             /* number of frames */

  int offset_skins;           /* offset skin data */
  int offset_st;              /* offset texture coordinate data */
  int offset_tris;            /* offset triangle data */
  int offset_frames;          /* offset frame data */
  int offset_glcmds;          /* offset OpenGL command data */
  int offset_end;             /* offset end of file */
};

struct md2_glcmd_t {
  float s;
  float t;
  int index;
};

/* Texture name */
struct md2_skin_t {
  char name[64];
};

/* Texture coords */
struct md2_texCoord_t {
  short s;
  short t;
};

/* Triangle info */
struct md2_triangle_t {
  unsigned short vertex[3];
  unsigned short st[3];
};

/* Compressed vertex */
struct md2_vertex_t {
  unsigned char v[3];
  unsigned char normalIndex;
};

/* Model frame */
struct md2_frame_t {
  vec3_t scale;
  vec3_t translate;
  char name[16];
  struct md2_vertex_t verts[1];
};

struct material {
	colour specular;
	colour diffuse;
	colour ambient;
	float specularCoeff;
	float diffuseCoeff;
	float ambientCoeff;
	float emissive;
	float poli;
	float opacity;
};

struct vertex {
	float x, y, z;
	float u, v;
};

struct face {
	int vertexindices[3];
	Vector normals[3];
	colour col[3];
	struct material mat;
};

// animation
struct md2_anim_t
{
    int     first_frame;            // first frame of the animation
    int     last_frame;             // number of frames
    int     fps;                    // number of frames per second
};

// animation state
struct md2_animState_t
{
    int     startframe;              // first frame
    int     endframe;                // last frame
    int     fps;                     // frame per second for this animation

    float   curr_time;               // current time
    float   old_time;                // old time
    float   interpol;                // percent of interpolation

    int     type;                    // animation type

    int     curr_frame;              // current frame
    int     next_frame;              // next frame
};

class Model {
private:
	/* md2 stuff */

	int num_frames;         // number of frames
    int num_vertices;            // number of vertices
    int num_glcmds;         // number of opengl commands

    vec3_t *vertices;        // vertex array
    int *glcmds;          // opengl command array
    int *lightnormals;    // normal index array

    unsigned int texid;            // texture id
    struct md2_animState_t anim;             // animation
    float scale;            // scale value

	Vector frustumCenter;
	float frustumRadius;

	static vec3_t anorms[162];
	static float anorms_dots[ 16 ][256];
	static GLuint skincount;
	static md2_anim_t animlist[21];

    void Animate( float time );
    void ProcessLighting();
    void Interpolate(vec3_t* vertlist);
    void RenderFrame(bool light = true);

protected:
	Vector pos;
	Vector up;
	Vector right;
	Vector view;
	Vector angle;
	
public:
	Model();
	~Model();
	void MakeFrustumSphere();
	bool LoadMD2(const char *filename);
	int LoadTex(char *filename);
	void DrawMD2(float time, bool wireframe, bool light = true);
	void DrawFrame( int frame, bool wireframe, bool light = true );
    void SetAnim( int type );
    void ScaleModel( float s ) { scale = s; }
	void DrawCoord();
	void Update();

	Vector GetView() {return view;};
	Vector GetPos() {return pos;};
	Vector GetRight() {return right;};
	Vector GetUp() {return up;};
	int GetAnimType() {return anim.type;};
	vec3_t* GetVertices() {return vertices;};
	int GetNum_Frames() {return num_frames;};
	int GetNum_Vertices() {return num_vertices;};
	float GetScale() {return scale;};
	Vector GetFrustumCenter() {return frustumCenter;};
	float GetFrustumRadius() {return frustumRadius;};
	void RotateX(float amount);
	void RotateY(float amount);
	void RotateZ(float amount);
	void MoveFwd(float amount);
	void MoveBack(float amount);
	void MoveRight(float amount);
	void MoveLeft(float amount);
	void MoveUp(float amount);
	void MoveDown(float amount);
	void SetModel(Vector pos, Vector view, Vector up, Vector right);
private:
	int textureCount;
	TexInfo** ti;
	int GetTextureIndex(char *filename);
	char* GetTextureName(int index);
public:
	unsigned int GetTextureID(char *filename);
	int LoadTexture(char *fname);
	void UnloadTexture(int id);
};